
from uperations.operation import Operation
import kernel
from ..utils import create_event

class MakeEvent(Operation):

    @staticmethod
    def name():
        return "make_event"

    @staticmethod
    def description():
        return "Create an event for an existing operation"

    def _parser(self, main_parser):
        main_parser.add_argument('library', help="Name of the library")
        main_parser.add_argument('operation', help="Name of the operation")
        main_parser.add_argument('event', help="Name of the event")
        return

    def is_publishable(self):
        return True

    def _run(self):
        operation = kernel.find_operation(self.args.library, self.args.operation)
        create_event(operation,self.args.event)
        return


from kernel.operation import Operation
import subprocess
from abc import abstractmethod

class CommandOperation(Operation):

    def _parser(self, main_parser):
        main_parser.add_argument('-c','--command', dest='command', help="Overwrite the command to be run")
        return main_parser

    def _before_start(self):
        super(CommandOperation, self)._before_start()
        return True

    def _on_running(self):
        super(CommandOperation, self)._on_running()
        return True

    def _on_error(self, exception):
        super(CommandOperation, self)._on_error(exception)
        return True

    def _on_completed(self):
        super(CommandOperation, self)._on_completed()
        return True

    @abstractmethod
    def _command(self):
        raise NotImplementedError

    def _run(self):
        base_command = self.args.command if not self.args.command == None else self._command()
        command = [base_command]+ self.unknown_args
        subprocess.check_output(command)



from uperations.operation import Operation
import subprocess

class Command(Operation):

    @staticmethod
    def name():
        return "Command"

    @staticmethod
    def description():
        return "Run any command from the terminal"

    def _parser(self, main_parser):
        main_parser.add_argument('command', help="Command to be run")
        return

    def _run(self):
        command = [self.args.command]+ self.unknown_args
        subprocess.check_output(command)


from uperations.operation import Operation

class OPERATIONTYPE(Operation):

    def _parser(self, main_parser):
        return main_parser

    def _before_start(self):
        super(OPERATIONTYPE, self)._before_start()
        return True

    def _on_running(self):
        super(OPERATIONTYPE, self)._on_running()
        return True

    def _on_error(self, exception):
        super(OPERATIONTYPE, self)._on_error(exception)
        return True

    def _on_completed(self):
        super(OPERATIONTYPE, self)._on_completed()
        return True



from vendors.uperations.operations.event import Event

class EVENTNAME(Event):

    @staticmethod
    def name():
        return "EVENTNAME"

    @staticmethod
    def description():
        return "No description provided for EVENTNAME"

    def __init__(self):
        super().__init__()
        return


import os
from uperations.operation import Operation
import kernel

class Publish(Operation):

    @staticmethod
    def name():
        return "Publish"

    @staticmethod
    def description():
        return "Publish resources files under operation"

    def _parser(self, main_parser):
        main_parser.add_argument('library',help="Library containing the operation")
        main_parser.add_argument('operation',help="Library containing the operation")
        return

    def _run(self):
        operation = kernel.find_operation(self.args.library, self.args.operation)
        if operation.is_publishable():
            operation.publish(os.path.join('resources',self.args.library,self.args.operation))
            return
        raise Exception("This operation is not publishable")




from uperations.operation import Operation

class HelloName(Operation):

    @staticmethod
    def name():
        return "hello_name"

    @staticmethod
    def description():
        return "Print hello to any name argument"

    def _parser(self, main_parser):
        main_parser.add_argument('name', help="Name to say hello to.")
        return

    def _run(self):
        print("Hello %s" % (self.args.name))




from uperations.library import Library
from .make_library import Makelibrary
from .make_operation import MakeOperation
from .make_operationtype import MakeOperationType
from .list_operation import Listoperation
from .yaml_to_json import Yamltojson
from .publish import Publish
from .command import Command
from .dockerfile import Dockerfile
from .helloworld import Helloworld
from .hello_name import HelloName

class Base(Library):

    @staticmethod
    def name():
        return "base"

    @staticmethod
    def description():
        return "General operations"

    def operations(self):
        return {
            'hello:world': Helloworld(self),
            'make:library': Makelibrary(self),
            'make:operation': MakeOperation(self),
            'make:operation:type': MakeOperationType(self),
            'list:operations': Listoperation(self),
            'yaml_to_json': Yamltojson(self),
            'publish': Publish(self),
            'command': Command(self),
            'dockerfile': Dockerfile(self),
            'hello': HelloName(self)
        }


from distutils.core import setup
from setuptools import find_packages

setup(
    name='uperations-base',
    version='0.0.2',
    packages=find_packages(),
    license='Creative Commons Attribution-Noncommercial-Share Alike license',
    long_description=open('README.rst').read(),
    author='Brice Aminou',
    author_email='brice.aminou@gmail.com',
    keywords='workflow tool',
    url='https://github.com/baminou/Uperations-Base'
)



from uperations.operation import Operation

class Helloworld(Operation):

    @staticmethod
    def name():
        return "hello_world"

    @staticmethod
    def description():
        return "Print Hello world!"

    def _parser(self, main_parser):
        return

    def _run(self):
        print("Hello world!")




import os
import shutil
from termcolor import cprint
import tempfile
from contracts import contract
from uperations.library import LibraryException, LibraryNotFound
from uperations.operation import OperationException

@contract(library_name='str', out_dir='str', returns='str')
def library_create(library_name, out_dir):
    """
    Create a new libary

    Args:
        library_name (str): Name of the library to create
        out_dir (str): Path of the directory where the library is going to be created

    Return:
        str: Path of the newly created library
    """

    if library_exists(library_name, out_dir):
        raise LibraryException("The library "+library_name+" already exists under "+out_dir)

    if not os.path.isdir(out_dir):
        raise FileNotFoundError(out_dir+" does not exist")

    library_new_path = os.path.join(out_dir, str(library_name).lower())
    library_new_init = os.path.join(library_new_path, '__init__.py')
    tmp_dir = os.path.join(tempfile.mkdtemp(),'tmp')
    tmp_init = os.path.join(tmp_dir, '__init__.py')

    shutil.copytree(os.path.join('operations','base','utils','templates','make_library'), tmp_dir)
    replace_placeholders_in_file(tmp_init, {'LIBRARYNAME':library_name.lower()})
    shutil.copytree(tmp_dir,library_new_path)
    shutil.rmtree(tmp_dir)

    cprint("Library successfully created under: %s" % library_new_init, 'green')

    return library_new_path

@contract(file_path='str', placeholders='dict')
def replace_placeholders_in_file(file_path, placeholders):
    """
    Replace a dictionary of placeholders in an existing file

    Args:
        file_path (str): Path of the file with the placeholders
        placeholders (dict): Dictionary containing list of placeholders as key, value {PLACEHOLDER: new_value}

    Raises:
        FileNotFoundError: file_path does not exist
        TypeError: file_path is not a string
        TypeError: placeholders is not a dict
    """
    if not os.path.isfile(file_path):
        raise FileNotFoundError(file_path+" does not exist")

    for key in placeholders:
        with open(file_path, 'r') as f:
            new_text = f.read().replace(key, placeholders[key])

        with open(file_path, 'w') as f:
            f.write(new_text)
    return

@contract(name='str',operations_dir='str', returns='bool')
def library_exists(name, operations_dir):
    """
    Check if a library exists in the specified directory

    Args:
        name: Name of the library to check existence
        operations_dir: Path of the operations directory

    Return:
        bool: True if the library exists, False otherwise

    Raises:
        FileNotFoundError: The operations directory does not exist
    """
    if not os.path.isdir(operations_dir):
        raise FileNotFoundError(operations_dir+" does not exist.")

    library_path = os.path.join(os.getcwd(), operations_dir, name)
    return os.path.isdir(library_path)

@contract(operations_dir="str",library_name="str",operation_name="str", returns="bool")
def operation_exists(operations_dir, library_name, operation_name):
    """
    Check if an operation exists or not

    Args:
        operations_dir (str): The directory where you want to create the operation
        library_name (str): Name of the library where to create the operation
        operation_name (str): Name of the operation to create in snake case

    Return:
        bool: True if operation exists, False otherwise

    Raises:
        FileNotFoundError: operations_dir path does not exist or is not found in the system
        LibraryNotFound: The library requested does not exist
    """
    if not os.path.isdir(operations_dir):
        raise FileNotFoundError(operations_dir)

    if not library_exists(library_name, operations_dir):
        raise LibraryNotFound(os.path.join(operations_dir,library_name))

    return os.path.isfile(os.path.join(operations_dir,library_name,operation_name,'__init__.py'))

@contract(operations_dir="str",library_name="str",operation_name="str", returns="str")
def operation_create(operations_dir, library_name, operation_name):
    """
    Create a new operation

    Args:
        library_name (str): Name of the library
        operations_dir (str): Directory of operations

    Return:
        str: Path of the new operation directory

    Raises:
        OperationException: The operation already exists
    """
    if not library_exists(library_name,operations_dir):
        library_create(library_name, operations_dir)

    if operation_exists(operations_dir, library_name, operation_name):
        raise OperationException(os.path.join(operations_dir,library_name, operation_name)+" already exists.")

    operation_new_path = os.path.join(operations_dir, library_name, operation_name)
    operation_new_init = os.path.join(operation_new_path, '__init__.py')
    tmp_dir = os.path.join(tempfile.mkdtemp(),'tmp')
    tmp_init = os.path.join(tmp_dir, '__init__.py')

    shutil.copytree(os.path.join('operations','base','utils','templates','make_operation'), tmp_dir)
    replace_placeholders_in_file(tmp_init, {'NEWOPERATION':to_camel_case(operation_name)})
    shutil.copytree(tmp_dir,operation_new_path)
    shutil.rmtree(tmp_dir)

    cprint("Operation successfully created under: %s" % operation_new_init, 'green')
    return operation_new_path

def create_event(operation, event_name):
    event_new_path = os.path.join(operation.events_dir(), event_name)
    event_new_init = os.path.join(event_new_path, '__init__.py')
    tmp_dir = os.path.join(tempfile.mkdtemp(),'tmp')
    tmp_init = os.path.join(tmp_dir, '__init__.py')

    shutil.copytree(os.path.join('operations','base','utils','templates','make_event'), tmp_dir)

    replace_placeholders_in_file(tmp_init, {'EVENTNAME':to_camel_case(event_name)})
    shutil.copytree(tmp_dir,event_new_path)
    shutil.rmtree(tmp_dir)
    cprint("Event successfully created under: %s" % event_new_init, 'green')
    return event_new_path

@contract(snake_str="str", returns='str')
def to_camel_case(snake_str):
    """
    Convert string from snake case to camel case

    Args:
        snake_str (str): A string in snake case

    Return:
        str: Input string in camel case
    """
    components = snake_str.split('_')
    return components[0].title() + ''.join(x.title() for x in components[1:])



from ..utils import *
import os
from contracts.interface import ContractNotRespected
import shutil
from vendors.uperations.operations.library import LibraryException

def test_library_exists_false(tmpdir):
    p = tmpdir.mkdir("tmp_operations")
    assert len(p.listdir()) == 0
    assert not library_exists('new_library',os.path.dirname(p))
    return

def test_library_exists_true(tmpdir):
    p = tmpdir.mkdir("tmp_operations")
    p.mkdir('new_library')
    assert library_exists('new_library',str(p))
    return

def test_library_exists_missing_dir(tmpdir):
    p = tmpdir.mkdir("tmp_operations")
    try:
        library_exists('new_library', os.path.join(p,'tmp'))
    except FileNotFoundError:
        assert True
    return

def test_library_exists_no_missing_dir(tmpdir):
    p = tmpdir.mkdir("tmp_operations")
    p.mkdir('tmp')
    try:
        library_exists('new_library', os.path.join(p,'tmp'))
        assert True
    except FileNotFoundError:
        assert False
    return

def test_library_exists_name_int(tmpdir):
    p = tmpdir.mkdir("tmp_operations")
    p.mkdir('tmp')

    try:
        library_exists(1,os.path.join(p,'tmp'))
        assert False
    except ContractNotRespected:
        assert True
    return

def test_library_exists_name_dict(tmpdir):
    p = tmpdir.mkdir("tmp_operations")
    p.mkdir('tmp')

    try:
        library_exists({},os.path.join(p,'tmp'))
        assert False
    except ContractNotRespected:
        assert True
    return

def test_library_exists_out_dir_dict(tmpdir):
    p = tmpdir.mkdir("tmp_operations")
    p.mkdir('tmp')

    try:
        library_exists("test",{})
        assert False
    except ContractNotRespected:
        assert True
    return

def test_library_exists_out_dir_int(tmpdir):
    p = tmpdir.mkdir("tmp_operations")
    p.mkdir('tmp')

    try:
        library_exists("test",0)
        assert False
    except ContractNotRespected:
        assert True
    shutil.rmtree(p)
    return

def test_replace_placeholders_in_file(tmpdir):
    p = tmpdir.mkdir("tmp_operations").join('__init__.py')
    placeholder = "TEST_PLACECHOLDER"
    p.write(placeholder)

    replace_placeholders_in_file(str(p),{placeholder:'new_value'})

    assert 'new_value' in p.read()
    assert not 'TEST_PLACECHOLDER' in p.read()
    os.remove(p)
    return

def test_library_create(tmpdir):
    p = tmpdir.mkdir("tmp_operations")

    template = p.mkdir('template')
    out_dir = p.mkdir('out_dir')
    open(os.path.join(template,'__init__.py'),'a').close()

    assert library_create('new_library',str(out_dir)) == os.path.join(out_dir,'new_library')
    try:
        library_create('new_library',str(out_dir)) == os.path.join(out_dir,'new_library')
        assert False
    except LibraryException:
        assert True
    shutil.rmtree(p)
    return

def test_to_camel_case():
    snake_str = "snake_str"
    assert to_camel_case(snake_str) == "SnakeStr"
    return
